/*
   $Id: CommandFreq.cpp 703 2013-01-25 09:06:49Z tfaraut $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <iostream>
#include <string>
using namespace std;

#include "ptypes.h"
#include "ParametersFreq.h"
#include "SuitableRegion.h"
#include "utils.h"
#include "ParametersFreq.h"
#include "CommandFreq.h"

#ifndef __LP64__
    #error "GLINT MAY BE COMPILED ONLY ON A 64 BITS PLATFORM"
#endif

extern RealLogger* logger;

/*////////////// Main ///////////////*/
int CommandFreq::Main(int argc,char* argv[])
{
	try
	{		
		Parameters& parameters = Parameters::Instance();
		ParametersInitializer& prm_init = _GetInitializer();
		prm_init.Init(argc, argv, *this);
		
		/* Initializing logger */
		string log_filename;
		string query_filename = parameters.GetQueryFilename().GetName(); //used only to determine the log_filename
			
		if (query_filename == "-")
			log_filename="glint_pipe.log";
		else
			log_filename = query_filename + ".log";
		
		auto_ptr<RealLogger> logger_auto = auto_ptr<RealLogger>(new RealLogger(log_filename,parameters.GetLogLevel(),LOGGER_OPT_AUTOFLUSH));
		logger = logger_auto.get();
		logger->SetLogFormat(no_stamp);

		SuitableRegion Sweetable;
		// Detecting the suitable regions from QueryFile
		Sweetable.Run();
				
	}
	catch ( exception& e)
	{
		cout << e.what() << endl;
		return 1;
	};
	return 0;
}

/*Utilisation du programme */
void CommandFreq::Usage() const
{
	PrintHeader();
	cerr << "Glint freq: Extract 'suitable' subregion with low occurancy sequence in the genome.\n";
	cerr << "           by A. Sagne, T. Faraut & E. Courcelle (INRA-CNRS)\n\n";
	cerr << "Usage:     glint freq [options] query \n\n";
	cerr << "\t         query: fasta or multifasta file, may compressed using Z,GZ,BZ2,LZMA,XZ algorithms \n\n";
	cerr << "\tOptions :\n";
	cerr << "\t\t --silent   : Less messages on stderr.\n";
	cerr << "\t\t --verbose  : More messages on stderr.\n";
	cerr << "\n\t\t========== DEFINE FREQUENCY SUBREGION =============\n";
	cerr << "\t\t --count_cutoff       : the constant which determine either a word is HIGH or LOW (the default is 6)\n";
	cerr << "\t\t --condemned_min_size : the minimum size of a condemned subregion (the default is 20)\n";
	cerr << "\t\t --conseqFreq         : the minimum proportion of frequent word for a condemned subregion (the default is 50)\n";
	cerr << "\t\t --suitable_min_size  : the minimum size of a suitable subregion (the default is 1000)\n";
	cerr << "\t\t --conseqUnFreq       : the maximum proportion of frequent word for a suitable subregion (the default is 20)\n";
	cerr << "\n\t\t========== REDIRECTING AND COMPRESSING THE OUTPUT ============\n";
	cerr << "\t\t -o filename, --output-file filename : output file (default stdout)\n";
	cerr << "\t\t When using the -o switch, the default is to use THE SAME compression algorithm as the input file\n";
	cerr << "\t\t This can be changed with the --compress switch:\n";
	cerr << "\t\t --compress Z|GZ|BZ2|LZMA|XZ Force one of those compression algorithms\n";
	cerr << "\t\t --compress any              Use the most efficient compression algorithm (depends on the programs installed on the host)\n";
		
	cerr << "\nDescription:\n";
	cerr << "\tThe program works in the following manner : \n";
	cerr << "\t\t- the index file is uploaded in memory as a factor tree like structure of k-words (k=size of the mask).\n";
	cerr << "\t\t- the query sequence is scanned in order to detect seeds using the factor tree like structure as an index.\n";
	cerr << "\t\t- Then high frequency subregions are defined as the 'condemned subregion'.\n";
	cerr << "\t\t- Low frequency subregions, defined as 'suitable subregion'.\n";
}

